@@ -48,6 +48,7 @@
 #include <nbdkit-filter.h>
 
 #include "cleanup.h"
+#include "utils.h"
 
 static uint64_t connections;
 static char *logfilename;
@@ -99,7 +100,7 @@ log_config_complete (nbdkit_next_config_complete *next, void *nxdata)
 
 /* Open the logfile. */
 static int
-log_get_ready (nbdkit_next_get_ready *next, void *nxdata)
+log_get_ready (nbdkit_next_get_ready *next, void *nxdata, int thread_model)
 {
   int fd;
 
@@ -133,6 +134,8 @@ log_get_ready (nbdkit_next_get_ready *next, void *nxdata)
 struct handle {
   uint64_t connection;
   uint64_t id;
+  char *exportname;
+  int tls;
 };
 
 /* Compute the next id number on the current connection. */
@@ -164,9 +167,14 @@ output (struct handle *h, const char *act, uint64_t id, const char *fmt, ...)
       snprintf (timestamp + s, sizeof timestamp - s, ".%06ld",
                 0L + tv.tv_usec);
     }
+#ifdef HAVE_FLOCKFILE
   flockfile (logfile);
-  fprintf (logfile, "%s connection=%" PRIu64 " %s ", timestamp, h->connection,
-           act);
+#endif
+  if (h)
+    fprintf (logfile, "%s connection=%" PRIu64 " %s ", timestamp,
+             h->connection, act);
+  else
+    fprintf (logfile, "%s %s ", timestamp, act);
   if (id)
     fprintf (logfile, "id=%" PRIu64 " ", id);
   va_start (args, fmt);
@@ -174,7 +182,9 @@ output (struct handle *h, const char *act, uint64_t id, const char *fmt, ...)
   va_end (args);
   fputc ('\n', logfile);
   fflush (logfile);
+#ifdef HAVE_FUNLOCKFILE
   funlockfile (logfile);
+#endif
 }
 
 /* Shared code for a nicer log of return value */
@@ -234,13 +244,55 @@ output_return (struct handle *h, const char *act, uint64_t id, int r, int *err)
   output (h, act, id, "return=%d (%s)", r, s);
 }
 
+/* List exports. */
+static int
+log_list_exports (nbdkit_next_list_exports *next, void *nxdata,
+                  int readonly, int default_only,
+                  struct nbdkit_exports *exports)
+{
+  static uint64_t id;
+  int r;
+  int err;
+
+  output (NULL, "ListExports", ++id, "readonly=%d default_only=%d ...",
+          readonly, default_only);
+  r = next (nxdata, readonly, default_only, exports);
+  if (r == -1) {
+    err = errno;
+    output_return (NULL, "...ListExports", id, r, &err);
+  }
+  else {
+    FILE *fp;
+    CLEANUP_FREE char *exports_str = NULL;
+    size_t i, n, len = 0;
+
+    fp = open_memstream (&exports_str, &len);
+    if (fp != NULL) {
+      n = nbdkit_exports_count (exports);
+      for (i = 0; i < n; ++i) {
+        struct nbdkit_export e = nbdkit_get_export (exports, i);
+        if (i > 0)
+          fprintf (fp, ", ");
+        shell_quote (e.name, fp);
+      }
+
+      fclose (fp);
+    }
+
+    output (NULL, "...ListExports", id, "exports=[%s] return=0",
+            exports_str ? exports_str : "(null)");
+  }
+  return r;
+}
+
 /* Open a connection. */
 static void *
-log_open (nbdkit_next_open *next, void *nxdata, int readonly)
+log_open (nbdkit_next_open *next, void *nxdata,
+          int readonly, const char *exportname, int is_tls)
 {
   struct handle *h;
 
-  if (next (nxdata, readonly) == -1)
+  if (next (nxdata, readonly, exportname) == -1)
     return NULL;
 
   h = malloc (sizeof *h);
@@ -249,6 +301,18 @@ log_open (nbdkit_next_open *next, void *nxdata, int readonly)
     return NULL;
   }
 
+  /* Save the exportname and tls state in the handle so we can display
+   * it in log_prepare.  We must take a copy because this string has a
+   * short lifetime.
+   */
+  h->exportname = strdup (exportname);
+  if (h->exportname == NULL) {
+    nbdkit_error ("strdup: %m");
+    free (h);
+    return NULL;
+  }
+  h->tls = is_tls;
+
   ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
   h->connection = ++connections;
   h->id = 0;
@@ -260,6 +324,7 @@ log_close (void *handle)
 {
   struct handle *h = handle;
 
+  free (h->exportname);
   free (h);
 }
 
@@ -268,6 +333,7 @@ log_prepare (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
              int readonly)
 {
   struct handle *h = handle;
+  const char *exportname = h->exportname;
   int64_t size = next_ops->get_size (nxdata);
   int w = next_ops->can_write (nxdata);
   int f = next_ops->can_flush (nxdata);
@@ -283,9 +349,9 @@ log_prepare (struct nbdkit_next_ops *next_ops, void *nxdata, void *handle,
       e < 0 || c < 0 || Z < 0)
     return -1;
 
-  output (h, "Connect", 0, "size=0x%" PRIx64 " write=%d flush=%d "
-          "rotational=%d trim=%d zero=%d fua=%d extents=%d cache=%d "
-          "fast_zero=%d", size, w, f, r, t, z, F, e, c, Z);
+  output (h, "Connect", 0, "export='%s' tls=%d size=0x%" PRIx64 " write=%d "
+          "flush=%d rotational=%d trim=%d zero=%d fua=%d extents=%d cache=%d "
+          "fast_zero=%d", exportname, h->tls, size, w, f, r, t, z, F, e, c, Z);
   return 0;
 }
 
@@ -461,6 +527,7 @@ static struct nbdkit_filter filter = {
   .config_help       = log_config_help,
   .unload            = log_unload,
   .get_ready         = log_get_ready,
+  .list_exports      = log_list_exports,
   .open              = log_open,
   .close             = log_close,
   .prepare           = log_prepare,
