@@ -76,47 +76,55 @@ extern "C" {
 #define NBDKIT_EXTENT_HOLE    (1<<0) /* Same as NBD_STATE_HOLE */
 #define NBDKIT_EXTENT_ZERO    (1<<1) /* Same as NBD_STATE_ZERO */
 
-extern void nbdkit_error (const char *msg, ...) ATTRIBUTE_FORMAT_PRINTF (1, 2);
-extern void nbdkit_verror (const char *msg, va_list args)
-  ATTRIBUTE_FORMAT_PRINTF (1, 0);
-extern void nbdkit_debug (const char *msg, ...) ATTRIBUTE_FORMAT_PRINTF (1, 2);
-extern void nbdkit_vdebug (const char *msg, va_list args)
-  ATTRIBUTE_FORMAT_PRINTF (1, 0);
+#define NBDKIT_EXTERN_DECL(ret, fn, args) extern ret fn args
 
-extern char *nbdkit_absolute_path (const char *path);
-extern int64_t nbdkit_parse_size (const char *str);
-extern int nbdkit_parse_bool (const char *str);
-extern int nbdkit_parse_int (const char *what, const char *str,
-                             int *r);
-extern int nbdkit_parse_unsigned (const char *what, const char *str,
-                                  unsigned *r);
-extern int nbdkit_parse_int8_t (const char *what, const char *str,
-                                int8_t *r);
-extern int nbdkit_parse_uint8_t (const char *what, const char *str,
-                                 uint8_t *r);
-extern int nbdkit_parse_int16_t (const char *what, const char *str,
-                                 int16_t *r);
-extern int nbdkit_parse_uint16_t (const char *what, const char *str,
-                                  uint16_t *r);
-extern int nbdkit_parse_int32_t (const char *what, const char *str,
-                                 int32_t *r);
-extern int nbdkit_parse_uint32_t (const char *what, const char *str,
-                                  uint32_t *r);
-extern int nbdkit_parse_int64_t (const char *what, const char *str,
-                                 int64_t *r);
-extern int nbdkit_parse_uint64_t (const char *what, const char *str,
-                                  uint64_t *r);
-extern int nbdkit_stdio_safe (void);
-extern int nbdkit_read_password (const char *value, char **password);
-extern char *nbdkit_realpath (const char *path);
-extern int nbdkit_nanosleep (unsigned sec, unsigned nsec);
-extern const char *nbdkit_export_name (void);
-extern int nbdkit_peer_name (struct sockaddr *addr, socklen_t *addrlen);
-extern void nbdkit_shutdown (void);
+NBDKIT_EXTERN_DECL (void, nbdkit_error,
+                    (const char *msg, ...) ATTRIBUTE_FORMAT_PRINTF (1, 2));
+NBDKIT_EXTERN_DECL (void, nbdkit_verror,
+                    (const char *msg, va_list args)
+                    ATTRIBUTE_FORMAT_PRINTF (1, 0));
+NBDKIT_EXTERN_DECL (void, nbdkit_debug,
+                    (const char *msg, ...) ATTRIBUTE_FORMAT_PRINTF (1, 2));
+NBDKIT_EXTERN_DECL (void, nbdkit_vdebug,
+                    (const char *msg, va_list args)
+                    ATTRIBUTE_FORMAT_PRINTF (1, 0));
+
+NBDKIT_EXTERN_DECL (char *, nbdkit_absolute_path, (const char *path));
+NBDKIT_EXTERN_DECL (int64_t, nbdkit_parse_size, (const char *str));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_bool, (const char *str));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_int,
+                    (const char *what, const char *str, int *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_unsigned,
+                    (const char *what, const char *str, unsigned *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_int8_t,
+                    (const char *what, const char *str, int8_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_uint8_t,
+                    (const char *what, const char *str, uint8_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_int16_t,
+                    (const char *what, const char *str, int16_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_uint16_t,
+                    (const char *what, const char *str, uint16_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_int32_t,
+                    (const char *what, const char *str, int32_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_uint32_t,
+                    (const char *what, const char *str, uint32_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_int64_t,
+                    (const char *what, const char *str, int64_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_parse_uint64_t,
+                    (const char *what, const char *str, uint64_t *r));
+NBDKIT_EXTERN_DECL (int, nbdkit_stdio_safe, (void));
+NBDKIT_EXTERN_DECL (int, nbdkit_read_password,
+                    (const char *value, char **password));
+NBDKIT_EXTERN_DECL (char *, nbdkit_realpath, (const char *path));
+NBDKIT_EXTERN_DECL (int, nbdkit_nanosleep, (unsigned sec, unsigned nsec));
+NBDKIT_EXTERN_DECL (int, nbdkit_peer_name,
+                    (struct sockaddr *addr, socklen_t *addrlen));
+NBDKIT_EXTERN_DECL (void, nbdkit_shutdown, (void));
 
 struct nbdkit_extents;
-extern int nbdkit_add_extent (struct nbdkit_extents *,
-                              uint64_t offset, uint64_t length, uint32_t type);
+NBDKIT_EXTERN_DECL (int, nbdkit_add_extent,
+                    (struct nbdkit_extents *,
+                     uint64_t offset, uint64_t length, uint32_t type));
 
 /* A static non-NULL pointer which can be used when you don't need a
  * per-connection handle.
