@@ -65,6 +65,19 @@
 #endif
 
 static char *filename = NULL;
+static char *directory = NULL;
+
+/* posix_fadvise mode: -1 = don't set it, or POSIX_FADV_*. */
+static int fadvise_mode =
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_NORMAL)
+  POSIX_FADV_NORMAL
+#else
+  -1
+#endif
+  ;
+
+/* cache mode */
+static enum { cache_default, cache_none } cache_mode = cache_default;
 
 /* Any callbacks using lseek must be protected by this lock. */
 static pthread_mutex_t lseek_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -82,21 +95,74 @@ static void
 file_unload (void)
 {
   free (filename);
+  free (directory);
 }
 
 /* Called for each key=value passed on the command line.  This plugin
- * only accepts file=<filename>, which is required.
+ * only accepts file=<filename> and dir=<dirname>, where exactly
+ * one is required.
  */
 static int
 file_config (const char *key, const char *value)
 {
+  /* See FILENAMES AND PATHS in nbdkit-plugin(3).
+   * Our use of nbdkit_realpath requires the destination to exist at
+   * startup; use nbdkit_absolute_path instead if we wanted to defer
+   * existence checks to the last possible moment.
+   */
   if (strcmp (key, "file") == 0) {
-    /* See FILENAMES AND PATHS in nbdkit-plugin(3). */
     free (filename);
     filename = nbdkit_realpath (value);
     if (!filename)
       return -1;
   }
+  else if (strcmp (key, "directory") == 0 ||
+           strcmp (key, "dir") == 0) {
+    free (directory);
+    directory = nbdkit_realpath (value);
+    if (!directory)
+      return -1;
+  }
+  else if (strcmp (key, "fadvise") == 0) {
+    /* As this is a hint, if the kernel doesn't support the feature
+     * ignore the parameter.
+     */
+    if (strcmp (value, "normal") == 0) {
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_NORMAL)
+      fadvise_mode = POSIX_FADV_NORMAL;
+#else
+      fadvise_mode = -1;
+#endif
+    }
+    else if (strcmp (value, "random") == 0) {
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_RANDOM)
+      fadvise_mode = POSIX_FADV_RANDOM;
+#else
+      fadvise_mode = -1;
+#endif
+    }
+    else if (strcmp (value, "sequential") == 0) {
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_SEQUENTIAL)
+      fadvise_mode = POSIX_FADV_SEQUENTIAL;
+#else
+      fadvise_mode = -1;
+#endif
+    }
+    else {
+      nbdkit_error ("unknown fadvise mode: %s", value);
+      return -1;
+    }
+  }
+  else if (strcmp (key, "cache") == 0) {
+    if (strcmp (value, "default") == 0)
+      cache_mode = cache_default;
+    else if (strcmp (value, "none") == 0)
+      cache_mode = cache_none;
+    else {
+      nbdkit_error ("unknown cache mode: %s", value);
+      return -1;
+    }
+  }
   else if (strcmp (key, "rdelay") == 0 ||
            strcmp (key, "wdelay") == 0) {
     nbdkit_error ("add --filter=delay on the command line");
@@ -110,13 +176,41 @@ file_config (const char *key, const char *value)
   return 0;
 }
 
-/* Check the user did pass a file=<FILENAME> parameter. */
+/* Check the user passed exactly one parameter. */
 static int
 file_config_complete (void)
 {
-  if (filename == NULL) {
-    nbdkit_error ("you must supply the file=<FILENAME> parameter "
-                  "after the plugin name on the command line");
+  int r;
+  struct stat sb;
+
+  if (!filename && !directory) {
+    nbdkit_error ("you must supply either [file=]<FILENAME> or "
+                  "dir=<DIRNAME> parameter after the plugin name "
+                  "on the command line");
+    return -1;
+  }
+  if (filename && directory) {
+    nbdkit_error ("file= and dir= cannot be used at the same time");
+    return -1;
+  }
+
+  /* Sanity check now, rather than waiting for first client open.
+   * See also comment in .config about use of nbdkit_realpath.
+   * Yes, this is a harmless TOCTTOU race.
+   */
+  if (filename) {
+    r = stat (filename, &sb);
+    if (r == 0 && S_ISDIR (sb.st_mode)) {
+      nbdkit_error ("use dir= to serve files within %s", filename);
+      return -1;
+    }
+    if (r == -1 || !(S_ISBLK (sb.st_mode) || S_ISREG (sb.st_mode))) {
+      nbdkit_error ("file is not regular or block device: %s", filename);
+      return -1;
+    }
+  }
+  else if (stat (directory, &sb) == -1 || !S_ISDIR (sb.st_mode)) {
+    nbdkit_error ("expecting a directory: %s", directory);
     return -1;
   }
 
@@ -124,7 +218,10 @@ file_config_complete (void)
 }
 
 #define file_config_help \
-  "file=<FILENAME>     (required) The filename to serve." \
+  "[file=]<FILENAME>     The filename to serve.\n" \
+  "dir=<DIRNAME>         A directory containing files to serve.\n" \
+  "cache=<MODE>          Set use of caching (default, none).\n" \
+  "fadise=<LEVEL>        Set fadvise hint (normal, random, sequential).\n" \
 
 /* Print some extra information about how the plugin was compiled. */
 static void
@@ -162,6 +259,24 @@ file_open (int readonly)
   struct handle *h;
   struct stat statbuf;
   int flags;
+  const char *file;
+  int dfd = -1;
+
+  if (directory) {
+    file = nbdkit_export_name ();
+    if (strchr (file, '/')) {
+      nbdkit_error ("exportname cannot contain /");
+      errno = EINVAL;
+      return NULL;
+    }
+    dfd = open (directory, O_RDONLY | O_DIRECTORY | O_CLOEXEC);
+    if (dfd == -1) {
+      nbdkit_error ("open %s: %m", directory);
+      return NULL;
+    }
+  }
+  else
+    file = filename;
 
   h = malloc (sizeof *h);
   if (h == NULL) {
@@ -175,17 +290,32 @@ file_open (int readonly)
   else
     flags |= O_RDWR;
 
-  h->fd = open (filename, flags);
+  h->fd = openat (dfd, file, flags);
   if (h->fd == -1) {
-    nbdkit_error ("open: %s: %m", filename);
+    nbdkit_error ("openat: %s: %m", file);
+    if (dfd != -1)
+      close (dfd);
+    free (h);
+    return NULL;
+  }
+  if (dfd != -1)
+    close (dfd);
+
+  if (fstat (h->fd, &statbuf) == -1) {
+    nbdkit_error ("fstat: %s: %m", file);
     free (h);
     return NULL;
   }
 
-  if (fstat (h->fd, &statbuf) == -1) {
-    nbdkit_error ("fstat: %s: %m", filename);
-    free (h);
-    return NULL;
+  if (fadvise_mode != -1) {
+    /* This is a hint so we ignore failures. */
+#ifdef HAVE_POSIX_FADVISE
+    int r = posix_fadvise (h->fd, 0, 0, fadvise_mode);
+    if (r == -1)
+      nbdkit_debug ("posix_fadvise: %s: %m (ignored)", file);
+#else
+    nbdkit_debug ("fadvise is not supported");
+#endif
   }
 
   if (S_ISBLK (statbuf.st_mode))
@@ -193,7 +323,7 @@ file_open (int readonly)
   else if (S_ISREG (statbuf.st_mode))
     h->is_block_device = false;
   else {
-    nbdkit_error ("file is not regular or block device: %s", filename);
+    nbdkit_error ("file is not regular or block device: %s", file);
     close (h->fd);
     free (h);
     return NULL;
@@ -203,7 +333,7 @@ file_open (int readonly)
 #ifdef BLKSSZGET
   if (h->is_block_device) {
     if (ioctl (h->fd, BLKSSZGET, &h->sector_size))
-      nbdkit_debug ("cannot get sector size: %s: %m", filename);
+      nbdkit_debug ("cannot get sector size: %s: %m", file);
   }
 #endif
 
@@ -338,6 +468,10 @@ file_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
             uint32_t flags)
 {
   struct handle *h = handle;
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_DONTNEED)
+  uint32_t orig_count = count;
+  uint64_t orig_offset = offset;
+#endif
 
   while (count > 0) {
     ssize_t r = pread (h->fd, buf, count, offset);
@@ -354,6 +488,12 @@ file_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
     offset += r;
   }
 
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_DONTNEED)
+  /* On Linux this will evict the pages we just read from the page cache. */
+  if (cache_mode == cache_none)
+    posix_fadvise (h->fd, orig_offset, orig_count, POSIX_FADV_DONTNEED);
+#endif
+
   return 0;
 }
 
@@ -364,6 +504,17 @@ file_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
 {
   struct handle *h = handle;
 
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_DONTNEED)
+  uint32_t orig_count = count;
+  uint64_t orig_offset = offset;
+
+  /* If cache=none we want to force pages we have just written to the
+   * file to be flushed to disk so we can immediately evict them from
+   * the page cache.
+   */
+  if (cache_mode == cache_none) flags |= NBDKIT_FLAG_FUA;
+#endif
+
   while (count > 0) {
     ssize_t r = pwrite (h->fd, buf, count, offset);
     if (r == -1) {
@@ -378,6 +529,12 @@ file_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
   if ((flags & NBDKIT_FLAG_FUA) && file_flush (handle, 0) == -1)
     return -1;
 
+#if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_DONTNEED)
+  /* On Linux this will evict the pages we just wrote from the page cache. */
+  if (cache_mode == cache_none)
+    posix_fadvise (h->fd, orig_offset, orig_count, POSIX_FADV_DONTNEED);
+#endif
+
   return 0;
 }
 
