@@ -46,6 +46,7 @@
 #include <semaphore.h>
 #include <poll.h>
 #include <fcntl.h>
+#include <sys/socket.h>
 
 #include <libnbd.h>
 
@@ -57,6 +58,15 @@
 #include "byte-swapping.h"
 #include "cleanup.h"
 #include "utils.h"
+#include "vector.h"
+
+DEFINE_VECTOR_TYPE(string_vector, const char *);
+
+#if !defined AF_VSOCK || !LIBNBD_HAVE_NBD_CONNECT_VSOCK
+#define USE_VSOCK 0
+#else
+#define USE_VSOCK 1
+#endif
 
 /* The per-transaction details */
 struct transaction {
@@ -84,8 +94,21 @@ static char *sockname;
 
 /* Connect to server via TCP socket */
 static const char *hostname;
+
+/* Valid with TCP or VSOCK */
 static const char *port;
 
+/* Connect to server via AF_VSOCK socket */
+static const char *raw_cid;
+static uint32_t cid;
+static uint32_t vport;
+
+/* Connect to a command. */
+static string_vector command = empty_vector;
+
+/* Connect to a socket file descriptor. */
+static int socket_fd = -1;
+
 /* Name of export on remote server, default '', ignored for oldstyle */
 static const char *export;
 
@@ -109,18 +132,16 @@ static void nbdplug_close_handle (struct handle *h);
 static void
 nbdplug_unload (void)
 {
-  if (shared)
+  if (shared && shared_handle)
     nbdplug_close_handle (shared_handle);
   free (sockname);
   free (tls_certificates);
   free (tls_psk);
+  free (command.ptr); /* the strings are statically allocated */
 }
 
-/* Called for each key=value passed on the command line.  This plugin
- * accepts socket=<sockname>, hostname=<hostname>/port=<port>, or
- * [uri=]<uri> (exactly one connection required), and optional
- * parameters export=<name>, retry=<n>, shared=<bool> and various
- * tls settings.
+/* Called for each key=value passed on the command line.  See
+ * nbdplug_config_help for the various keys recognized.
  */
 static int
 nbdplug_config (const char *key, const char *value)
@@ -138,8 +159,25 @@ nbdplug_config (const char *key, const char *value)
     hostname = value;
   else if (strcmp (key, "port") == 0)
     port = value;
+  else if (strcmp (key, "vsock") == 0 ||
+           strcmp (key, "cid") == 0)
+    raw_cid = value;
   else if (strcmp (key, "uri") == 0)
     uri = value;
+  else if (strcmp (key, "command") == 0 || strcmp (key, "arg") == 0) {
+    if (string_vector_append (&command, value) == -1) {
+      nbdkit_error ("realloc: %m");
+      return -1;
+    }
+  }
+  else if (strcmp (key, "socket-fd") == 0) {
+    if (nbdkit_parse_int ("socket-fd", value, &socket_fd) == -1)
+      return -1;
+    if (socket_fd < 0) {
+      nbdkit_error ("socket-fd must be >= 0");
+      return -1;
+    }
+  }
   else if (strcmp (key, "export") == 0)
     export = value;
   else if (strcmp (key, "retry") == 0) {
@@ -195,23 +233,25 @@ nbdplug_config (const char *key, const char *value)
 static int
 nbdplug_config_complete (void)
 {
-  int c = !!sockname + !!hostname + !!uri;
+  int c = !!sockname + !!hostname + !!uri +
+    (command.size > 0) + (socket_fd >= 0) + !!raw_cid;
 
   /* Check the user passed exactly one connection parameter. */
   if (c > 1) {
-    nbdkit_error ("cannot mix Unix ‘socket’, TCP ‘hostname’/‘port’ "
-                  "and ‘uri’ parameters");
+    nbdkit_error ("cannot mix Unix ‘socket’, TCP ‘hostname’/‘port’, ‘vsock’, "
+                  "‘command’, ‘socket-fd’ and ‘uri’ parameters");
     return -1;
   }
   if (c == 0) {
-    nbdkit_error ("exactly one of ‘socket’, ‘hostname’ "
-                  "and ‘uri’ parameters must be specified");
+    nbdkit_error ("exactly one of ‘socket’, ‘hostname’, ‘vsock’, ‘command’, "
+                  "‘socket-fd’ and ‘uri’ parameters must be specified");
     return -1;
   }
 
-  /* Port, if present, should only be used with hostname. */
-  if (port && !hostname) {
-    nbdkit_error ("‘port’ parameter should only be used with ‘hostname’");
+  /* Port, if present, should only be used with hostname or vsock. */
+  if (port && !(hostname || raw_cid)) {
+    nbdkit_error ("‘port’ parameter should only be used with ‘hostname’ or "
+                  "‘vsock’");
     return -1;
   }
 
@@ -241,6 +281,29 @@ nbdplug_config_complete (void)
     if (!port)
       port = "10809";
   }
+  else if (raw_cid) {
+#if !USE_VSOCK
+    nbdkit_error ("libnbd was compiled without vsock support");
+    return -1;
+#else
+    if (!port)
+      port = "10809";
+    if (nbdkit_parse_uint32_t ("vsock_cid", raw_cid, &cid) == -1 ||
+        nbdkit_parse_uint32_t ("port", port, &vport) == -1)
+      return -1;
+#endif
+  }
+  else if (command.size > 0) {
+    /* Add NULL sentinel to the command. */
+    if (string_vector_append (&command, NULL) == -1) {
+      nbdkit_error ("realloc: %m");
+      return -1;
+    }
+    shared = true;
+  }
+  else if (socket_fd >= 0) {
+    shared = true;
+  }
   else {
     abort ();         /* can't happen, if checks above were correct */
   }
@@ -266,8 +329,15 @@ nbdplug_config_complete (void)
     }
     nbd_close (nbd);
   }
+  return 0;
+}
 
-  /* Create the shared connection. */
+/* Create the shared connection.  Because this may create a background
+ * thread it must be done after we fork.
+ */
+static int
+nbdplug_after_fork (void)
+{
   if (shared && (shared_handle = nbdplug_open_handle (false)) == NULL)
     return -1;
   return 0;
@@ -277,7 +347,11 @@ nbdplug_config_complete (void)
   "[uri=]<URI>            URI of an NBD socket to connect to (if supported).\n" \
   "socket=<SOCKNAME>      The Unix socket to connect to.\n" \
   "hostname=<HOST>        The hostname for the TCP socket to connect to.\n" \
-  "port=<PORT>            TCP port or service name to use (default 10809).\n" \
+  "port=<PORT>            TCP/VSOCK port or service name to use (default 10809).\n" \
+  "vsock=<CID>            The cid for the VSOCK socket to connect to.\n" \
+  "command=<COMMAND>      Command to run.\n" \
+  "arg=<ARG>              Parameters for command.\n" \
+  "socket-fd=<FD>         Socket file descriptor to connect to.\n" \
   "export=<NAME>          Export name to connect to (default \"\").\n" \
   "retry=<N>              Retry connection up to N seconds (default 0).\n" \
   "shared=<BOOL>          True to share one server connection among all clients,\n" \
@@ -300,6 +374,7 @@ nbdplug_dump_plugin (void)
   printf ("libnbd_version=%s\n", nbd_get_version (nbd));
   printf ("libnbd_tls=%d\n", nbd_supports_tls (nbd));
   printf ("libnbd_uri=%d\n", nbd_supports_uri (nbd));
+  printf ("libnbd_vsock=%d\n", USE_VSOCK);
   nbd_close (nbd);
 }
 
@@ -478,28 +553,38 @@ nbdplug_open_handle (int readonly)
  retry:
   h->nbd = nbd_create ();
   if (!h->nbd)
-    goto err;
+    goto errnbd;
   if (nbd_set_export_name (h->nbd, export) == -1)
-    goto err;
+    goto errnbd;
   if (nbd_add_meta_context (h->nbd, LIBNBD_CONTEXT_BASE_ALLOCATION) == -1)
-    goto err;
+    goto errnbd;
   if (nbd_set_tls (h->nbd, tls) == -1)
-    goto err;
+    goto errnbd;
   if (tls_certificates &&
       nbd_set_tls_certificates (h->nbd, tls_certificates) == -1)
-    goto err;
+    goto errnbd;
   if (tls_verify >= 0 && nbd_set_tls_verify_peer (h->nbd, tls_verify) == -1)
-    goto err;
+    goto errnbd;
   if (tls_username && nbd_set_tls_username (h->nbd, tls_username) == -1)
-    goto err;
+    goto errnbd;
   if (tls_psk && nbd_set_tls_psk_file (h->nbd, tls_psk) == -1)
-    goto err;
+    goto errnbd;
   if (uri)
     r = nbd_connect_uri (h->nbd, uri);
   else if (sockname)
     r = nbd_connect_unix (h->nbd, sockname);
   else if (hostname)
     r = nbd_connect_tcp (h->nbd, hostname, port);
+  else if (raw_cid)
+#if !USE_VSOCK
+    abort ();
+#else
+    r = nbd_connect_vsock (h->nbd, cid, vport);
+#endif
+  else if (command.size > 0)
+    r = nbd_connect_systemd_socket_activation (h->nbd, (char **) command.ptr);
+  else if (socket_fd >= 0)
+    r = nbd_connect_socket (h->nbd, socket_fd);
   else
     abort ();
   if (r == -1) {
@@ -509,7 +594,7 @@ nbdplug_open_handle (int readonly)
       sleep (1);
       goto retry;
     }
-    goto err;
+    goto errnbd;
   }
 
   if (readonly)
@@ -523,10 +608,11 @@ nbdplug_open_handle (int readonly)
 
   return h;
 
+ errnbd:
+  nbdkit_error ("failure while creating nbd handle: %s", nbd_get_error ());
  err:
   close (h->fds[0]);
   close (h->fds[1]);
-  nbdkit_error ("failure while creating nbd handle: %s", nbd_get_error ());
   if (h->nbd)
     nbd_close (h->nbd);
   free (h);
@@ -859,6 +945,7 @@ static struct nbdkit_plugin plugin = {
   .config_complete    = nbdplug_config_complete,
   .config_help        = nbdplug_config_help,
   .magic_config_key   = "uri",
+  .after_fork         = nbdplug_after_fork,
   .dump_plugin        = nbdplug_dump_plugin,
   .open               = nbdplug_open,
   .close              = nbdplug_close,
