@@ -120,6 +120,7 @@ static value config_complete_fn;
 static value thread_model_fn;
 
 static value get_ready_fn;
+static value after_fork_fn;
 
 static value preconnect_fn;
 static value open_fn;
@@ -272,6 +273,25 @@ get_ready_wrapper (void)
   CAMLreturnT (int, 0);
 }
 
+static int
+after_fork_wrapper (void)
+{
+  CAMLparam0 ();
+  CAMLlocal1 (rv);
+
+  caml_leave_blocking_section ();
+
+  rv = caml_callback_exn (after_fork_fn, Val_unit);
+  if (Is_exception_result (rv)) {
+    nbdkit_error ("%s", caml_format_exception (Extract_exception (rv)));
+    caml_enter_blocking_section ();
+    CAMLreturnT (int, -1);
+  }
+
+  caml_enter_blocking_section ();
+  CAMLreturnT (int, 0);
+}
+
 static int
 preconnect_wrapper (int readonly)
 {
@@ -833,6 +853,7 @@ SET(config_complete)
 SET(thread_model)
 
 SET(get_ready)
+SET(after_fork)
 
 SET(preconnect)
 SET(open)
@@ -876,6 +897,7 @@ remove_roots (void)
   REMOVE (thread_model);
 
   REMOVE (get_ready);
+  REMOVE (after_fork);
 
   REMOVE (preconnect);
   REMOVE (open);
