@@ -51,6 +51,19 @@
 
 #include "cleanup.h"
 
+/* All callbacks that want to call any Py* function should use this
+ * macro.  See
+ * https://docs.python.org/3/c-api/init.html#non-python-created-threads
+ */
+#define ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE            \
+  __attribute__((cleanup (cleanup_release)))            \
+  PyGILState_STATE gstate = PyGILState_Ensure()
+static inline void
+cleanup_release (PyGILState_STATE *gstateptr)
+{
+  PyGILState_Release (*gstateptr);
+}
+
 /* XXX Apparently global state is technically wrong in Python 3, see:
  *
  * https://www.python.org/dev/peps/pep-3121/
@@ -65,24 +78,6 @@ static int py_api_version = 1;
 
 static __thread int last_error;
 
-static PyObject *
-set_error (PyObject *self, PyObject *args)
-{
-  int err;
-
-  if (!PyArg_ParseTuple (args, "i", &err))
-    return NULL;
-  nbdkit_set_error (err);
-  last_error = err;
-  Py_RETURN_NONE;
-}
-
-static PyMethodDef NbdkitMethods[] = {
-  { "set_error", set_error, METH_VARARGS,
-    "Store an errno value prior to throwing an exception" },
-  { NULL }
-};
-
 /* Is a callback defined? */
 static int
 callback_defined (const char *name, PyObject **obj_rtn)
@@ -211,6 +206,62 @@ check_python_failure (const char *callback)
   return 0;
 }
 
+/* Functions in the virtual nbdkit.* module. */
+
+/* nbdkit.debug */
+static PyObject *
+debug (PyObject *self, PyObject *args)
+{
+  const char *msg;
+
+  if (!PyArg_ParseTuple (args, "s", &msg))
+    return NULL;
+  nbdkit_debug ("%s", msg);
+  Py_RETURN_NONE;
+}
+
+/* nbdkit.export_name */
+static PyObject *
+export_name (PyObject *self, PyObject *args)
+{
+  const char *s = nbdkit_export_name ();
+
+  if (!s) {
+    /* Unfortunately we lose the actual error. XXX */
+    PyErr_SetString (PyExc_RuntimeError, "nbdkit.export_name failed");
+    return NULL;
+  }
+
+  /* NBD spec says that the export name should be UTF-8, so this
+   * ought to work, and if it fails the client gave us a bad export
+   * name which should turn into an exception.
+   */
+  return PyUnicode_FromString (s);
+}
+
+/* nbdkit.set_error */
+static PyObject *
+set_error (PyObject *self, PyObject *args)
+{
+  int err;
+
+  if (!PyArg_ParseTuple (args, "i", &err))
+    return NULL;
+  nbdkit_set_error (err);
+  last_error = err;
+  Py_RETURN_NONE;
+}
+
+static PyMethodDef NbdkitMethods[] = {
+  { "debug", debug, METH_VARARGS,
+    "Print a debug message" },
+  { "export_name", export_name, METH_VARARGS,
+    "Return the optional export name negotiated with the client" },
+  { "set_error", set_error, METH_VARARGS,
+    "Store an errno value prior to throwing an exception" },
+  { NULL }
+};
+
 static struct PyModuleDef moduledef = {
   PyModuleDef_HEAD_INIT,
   "nbdkit",
@@ -266,23 +317,30 @@ create_nbdkit_module (void)
   return m;
 }
 
+static PyThreadState *tstate;
+
 static void
 py_load (void)
 {
   PyImport_AppendInittab ("nbdkit", create_nbdkit_module);
   Py_Initialize ();
+  tstate = PyEval_SaveThread ();
 }
 
 static void
 py_unload (void)
 {
-  Py_XDECREF (module);
-  Py_Finalize ();
+  if (tstate) {
+    PyEval_RestoreThread (tstate);
+    Py_XDECREF (module);
+    Py_Finalize ();
+  }
 }
 
 static void
 py_dump_plugin (void)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   PyObject *fn;
   PyObject *r;
 
@@ -330,6 +388,7 @@ get_py_api_version (void)
 static int
 py_config (const char *key, const char *value)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   int fd;
   FILE *fp;
   PyObject *modname;
@@ -418,6 +477,7 @@ py_config (const char *key, const char *value)
 static int
 py_config_complete (void)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   PyObject *fn;
   PyObject *r;
 
@@ -434,9 +494,32 @@ py_config_complete (void)
   return 0;
 }
 
+static int
+py_thread_model (void)
+{
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
+  PyObject *fn;
+  PyObject *r;
+  int ret = NBDKIT_THREAD_MODEL_SERIALIZE_ALL_REQUESTS;
+
+  if (script && callback_defined ("thread_model", &fn)) {
+    PyErr_Clear ();
+
+    r = PyObject_CallObject (fn, NULL);
+    Py_DECREF (fn);
+    if (check_python_failure ("thread_model") == -1)
+      return -1;
+    ret = PyLong_AsLong (r);
+    Py_DECREF (r);
+  }
+
+  return ret;
+}
+
 static int
 py_get_ready (void)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   PyObject *fn;
   PyObject *r;
 
@@ -461,6 +544,7 @@ struct handle {
 static void *
 py_open (int readonly)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   PyObject *fn;
   struct handle *h;
 
@@ -493,6 +577,7 @@ py_open (int readonly)
 static void
 py_close (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -513,6 +598,7 @@ py_close (void *handle)
 static int64_t
 py_get_size (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -542,6 +628,7 @@ static int
 py_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
           uint32_t flags)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -604,6 +691,7 @@ static int
 py_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
            uint32_t flags)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -640,6 +728,7 @@ py_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
 static int
 py_flush (void *handle, uint32_t flags)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -672,6 +761,7 @@ py_flush (void *handle, uint32_t flags)
 static int
 py_trim (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -704,6 +794,7 @@ py_trim (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 static int
 py_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -750,6 +841,7 @@ py_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 static int
 py_cache (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -809,36 +901,42 @@ boolean_callback (void *handle, const char *can_fn, const char *plain_fn)
 static int
 py_is_rotational (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   return boolean_callback (handle, "is_rotational", NULL);
 }
 
 static int
 py_can_multi_conn (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   return boolean_callback (handle, "can_multi_conn", NULL);
 }
 
 static int
 py_can_write (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   return boolean_callback (handle, "can_write", "pwrite");
 }
 
 static int
 py_can_flush (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   return boolean_callback (handle, "can_flush", "flush");
 }
 
 static int
 py_can_trim (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   return boolean_callback (handle, "can_trim", "trim");
 }
 
 static int
 py_can_zero (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
 
   if (h->can_zero >= 0)
@@ -849,6 +947,7 @@ py_can_zero (void *handle)
 static int
 py_can_fast_zero (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   int r;
 
   if (callback_defined ("can_fast_zero", NULL))
@@ -866,6 +965,7 @@ py_can_fast_zero (void *handle)
 static int
 py_can_fua (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -894,6 +994,7 @@ py_can_fua (void *handle)
 static int
 py_can_cache (void *handle)
 {
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
   struct handle *h = handle;
   PyObject *fn;
   PyObject *r;
@@ -919,11 +1020,104 @@ py_can_cache (void *handle)
     return NBDKIT_CACHE_NONE;
 }
 
+static int
+py_can_extents (void *handle)
+{
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
+  return boolean_callback (handle, "can_extents", "extents");
+}
+
+static int
+py_extents (void *handle, uint32_t count, uint64_t offset,
+            uint32_t flags, struct nbdkit_extents *extents)
+{
+  ACQUIRE_PYTHON_GIL_FOR_CURRENT_SCOPE;
+  struct handle *h = handle;
+  PyObject *fn;
+  PyObject *r;
+  PyObject *iter, *t;
+  size_t size;
+
+  if (callback_defined ("extents", &fn)) {
+    PyErr_Clear ();
+
+    r = PyObject_CallFunction (fn, "OiLI", h->py_h, count, offset, flags);
+    Py_DECREF (fn);
+    if (check_python_failure ("extents") == -1)
+      return -1;
+
+    iter = PyObject_GetIter (r);
+    if (iter == NULL) {
+      nbdkit_error ("extents method did not return "
+                    "something which is iterable");
+      Py_DECREF (r);
+      return -1;
+    }
+
+    size = 0;
+    while ((t = PyIter_Next (iter)) != NULL) {
+      PyObject *py_offset, *py_length, *py_type;
+      uint64_t extent_offset, extent_length;
+      uint32_t extent_type;
+
+      size++;
+
+      if (!PyTuple_Check (t) || PyTuple_Size (t) != 3) {
+        nbdkit_error ("extents method did not return an iterable of 3-tuples");
+        Py_DECREF (iter);
+        Py_DECREF (r);
+        return -1;
+      }
+      py_offset = PyTuple_GetItem (t, 0);
+      py_length = PyTuple_GetItem (t, 1);
+      py_type = PyTuple_GetItem (t, 2);
+      extent_offset = PyLong_AsUnsignedLongLong (py_offset);
+      extent_length = PyLong_AsUnsignedLongLong (py_length);
+      extent_type = PyLong_AsUnsignedLong (py_type);
+      if (check_python_failure ("PyLong") == -1) {
+        Py_DECREF (iter);
+        Py_DECREF (r);
+        return -1;
+      }
+      if (nbdkit_add_extent (extents,
+                             extent_offset, extent_length, extent_type) == -1) {
+        Py_DECREF (iter);
+        Py_DECREF (r);
+        return -1;
+      }
+    }
+
+    if (size < 1) {
+      nbdkit_error ("extents method cannot return an empty list");
+      Py_DECREF (iter);
+      Py_DECREF (r);
+      return -1;
+    }
+
+    Py_DECREF (iter);
+    Py_DECREF (r);
+  }
+  else {
+    /* Do the same as the core server: synthesize a fully allocated
+     * extent covering the whole range.
+     */
+    if (nbdkit_add_extent (extents,
+                           offset, count, 0 /* allocated data */) == -1)
+      return -1;
+  }
+
+  return 0;
+}
+
 #define py_config_help \
   "script=<FILENAME>     (required) The Python plugin to run.\n" \
   "[other arguments may be used by the plugin that you load]"
 
-#define THREAD_MODEL NBDKIT_THREAD_MODEL_SERIALIZE_ALL_REQUESTS
+/* This is the maximum possible, but the default for plugins is
+ * NBDKIT_THREAD_MODEL_SERIALIZE_ALL_REQUESTS.  Plugins can override
+ * that by providing a thread_model() function.
+ */
+#define THREAD_MODEL NBDKIT_THREAD_MODEL_PARALLEL
 
 static struct nbdkit_plugin plugin = {
   .name              = "python",
@@ -937,6 +1131,7 @@ static struct nbdkit_plugin plugin = {
   .config_complete   = py_config_complete,
   .config_help       = py_config_help,
 
+  .thread_model      = py_thread_model,
   .get_ready         = py_get_ready,
 
   .open              = py_open,
@@ -952,6 +1147,7 @@ static struct nbdkit_plugin plugin = {
   .can_fast_zero     = py_can_fast_zero,
   .can_fua           = py_can_fua,
   .can_cache         = py_can_cache,
+  .can_extents       = py_can_extents,
 
   .pread             = py_pread,
   .pwrite            = py_pwrite,
@@ -959,6 +1155,7 @@ static struct nbdkit_plugin plugin = {
   .trim              = py_trim,
   .zero              = py_zero,
   .cache             = py_cache,
+  .extents           = py_extents,
 };
 
 NBDKIT_REGISTER_PLUGIN (plugin)
