@@ -165,6 +165,31 @@ sh_get_ready (void)
   }
 }
 
+int
+sh_after_fork (void)
+{
+  const char *method = "after_fork";
+  const char *script = get_script (method);
+  const char *args[] = { script, method, NULL };
+
+  switch (call (args)) {
+  case OK:
+  case MISSING:
+    return 0;
+
+  case ERROR:
+    return -1;
+
+  case RET_FALSE:
+    nbdkit_error ("%s: %s method returned unexpected code (3/false)",
+                  script, method);
+    errno = EIO;
+    return -1;
+
+  default: abort ();
+  }
+}
+
 int
 sh_preconnect (int readonly)
 {
@@ -173,7 +198,6 @@ sh_preconnect (int readonly)
   const char *args[] =
     { script, method,
       readonly ? "true" : "false",
-      nbdkit_export_name () ? : "",
       NULL };
 
   switch (call (args)) {
@@ -210,6 +234,7 @@ sh_open (int readonly)
     { script, method,
       readonly ? "true" : "false",
       nbdkit_export_name () ? : "",
+      nbdkit_is_tls () ? "true" : "false",
       NULL };
   struct sh_handle *h = malloc (sizeof *h);
 
@@ -239,8 +264,11 @@ sh_open (int readonly)
      */
     free (h->h);
     h->h = strdup ("");
-    if (h->h == NULL)
+    if (h->h == NULL) {
       nbdkit_error ("strdup: %m");
+      free (h);
+      return NULL;
+    }
     return h;
 
   case ERROR:
