@@ -35,6 +35,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <inttypes.h>
 #include <string.h>
 #include <unistd.h>
@@ -331,6 +332,8 @@ new_connection (int sockin, int sockout, int nworkers)
 static void
 free_connection (struct connection *conn)
 {
+  struct backend *b;
+
   if (!conn)
     return;
 
@@ -356,7 +359,14 @@ free_connection (struct connection *conn)
   pthread_mutex_destroy (&conn->write_lock);
   pthread_mutex_destroy (&conn->status_lock);
 
+  free (conn->exportname_from_set_meta_context);
+  free (conn->exportname);
+
+  /* This is needed in order to free a field in struct handle. */
+  for_each_backend (b)
+    reset_handle (get_handle (conn, b->i));
   free (conn->handles);
+
   free (conn);
   threadlocal_set_conn (NULL);
 }
