@@ -45,6 +45,7 @@
 #include "nbdkit-filter.h"
 #include "cleanup.h"
 #include "nbd-protocol.h"
+#include "unix-path-max.h"
 #include "vector.h"
 
 /* Define unlikely macro, but only for GCC.  These are used to move
@@ -58,12 +59,6 @@
 #define if_verbose if (verbose)
 #endif
 
-#ifdef __APPLE__
-#define UNIX_PATH_MAX 104
-#else
-#define UNIX_PATH_MAX 108
-#endif
-
 #if HAVE_VALGRIND
 # include <valgrind.h>
 /* http://valgrind.org/docs/manual/faq.html#faq.unhelpful */
@@ -113,7 +108,7 @@ enum log_to {
 };
 
 extern struct debug_flag *debug_flags;
-extern const char *exportname;
+extern const char *export_name;
 extern bool foreground;
 extern const char *ipaddr;
 extern enum log_to log_to;
@@ -207,6 +202,8 @@ struct handle {
 
   unsigned char state;  /* Bitmask of HANDLE_* values */
 
+  char *default_exportname;
+
   uint64_t exportsize;
   int can_write;
   int can_flush;
@@ -225,6 +222,8 @@ reset_handle (struct handle *h)
 {
   h->handle = NULL;
   h->state = 0;
+  free (h->default_exportname);
+  h->default_exportname = NULL;
   h->exportsize = -1;
   h->can_write = -1;
   h->can_flush = -1;
@@ -252,8 +251,6 @@ struct connection {
   struct handle *handles;       /* One per plugin and filter. */
   size_t nr_handles;
 
-  char exportname[NBD_MAX_STRING + 1];
-  uint32_t exportnamelen;
   uint32_t cflags;
   uint16_t eflags;
   bool handshake_complete;
@@ -261,6 +258,9 @@ struct connection {
   bool structured_replies;
   bool meta_context_base_allocation;
 
+  char *exportname_from_set_meta_context;
+  char *exportname;
+
   int sockin, sockout;
   connection_recv_function recv;
   connection_send_function send;
@@ -279,8 +279,9 @@ extern int connection_set_status (int value);
 
 /* protocol-handshake.c */
 extern int protocol_handshake (void);
-extern int protocol_common_open (uint64_t *exportsize, uint16_t *flags)
-  __attribute__((__nonnull__ (1, 2)));
+extern int protocol_common_open (uint64_t *exportsize, uint16_t *flags,
+                                 const char *exportname)
+  __attribute__((__nonnull__ (1, 2, 3)));
 
 /* protocol-handshake-oldstyle.c */
 extern int protocol_handshake_oldstyle (void);
@@ -362,8 +363,12 @@ struct backend {
   void (*config_complete) (struct backend *);
   const char *(*magic_config_key) (struct backend *);
   void (*get_ready) (struct backend *);
+  void (*after_fork) (struct backend *);
   int (*preconnect) (struct backend *, int readonly);
-  void *(*open) (struct backend *, int readonly);
+  int (*list_exports) (struct backend *, int readonly, int default_only,
+                       struct nbdkit_exports *exports);
+  void *(*open) (struct backend *, int readonly, const char *exportname,
+                 int is_tls);
   int (*prepare) (struct backend *, void *handle, int readonly);
   int (*finalize) (struct backend *, void *handle);
   void (*close) (struct backend *, void *handle);
@@ -407,8 +412,17 @@ extern void backend_load (struct backend *b, const char *name,
 extern void backend_unload (struct backend *b, void (*unload) (void))
   __attribute__((__nonnull__ (1)));
 
-extern int backend_open (struct backend *b, int readonly)
-  __attribute__((__nonnull__ (1)));
+extern int backend_list_exports (struct backend *b, int readonly,
+                                 int default_only,
+                                 struct nbdkit_exports *exports)
+  __attribute__((__nonnull__ (1, 4)));
+/* exportname is only valid for this call and almost certainly will be
+ * freed on return of this function, so backends must save the
+ * exportname if they need to refer to it later.
+ */
+extern int backend_open (struct backend *b,
+                         int readonly, const char *exportname)
+  __attribute__((__nonnull__ (1, 3)));
 extern int backend_prepare (struct backend *b)
   __attribute__((__nonnull__ (1)));
 extern int backend_finalize (struct backend *b)
@@ -419,8 +433,9 @@ extern bool backend_valid_range (struct backend *b,
                                  uint64_t offset, uint32_t count)
   __attribute__((__nonnull__ (1)));
 
-extern int backend_reopen (struct backend *b, int readonly)
-  __attribute__((__nonnull__ (1)));
+extern int backend_reopen (struct backend *b,
+                           int readonly, const char *exportname)
+  __attribute__((__nonnull__ (1, 3)));
 extern int64_t backend_get_size (struct backend *b)
   __attribute__((__nonnull__ (1)));
 extern int backend_can_write (struct backend *b)
