@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2017-2019 Red Hat Inc.
+# Copyright (C) 2017-2020 Red Hat Inc.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -35,5 +35,4 @@ set -e
 
 # We only really care that the plugin compiled, so we don't
 # need to run it properly.
-libs=./.libs
-nbdkit $libs/test-ansi-c-plugin.so --version
+nbdkit .libs/test-ansi-c-plugin.so --version
