@@ -31,7 +31,7 @@
 # SUCH DAMAGE.
 
 # Unfortunately the output of this test depends on the PAGE_SIZE
-# defined in common/sparse/sparse.c and would change (breaking the
+# defined in common/allocators/sparse.c and would change (breaking the
 # test) if we ever changed that definition.
 
 source ./functions.sh
@@ -51,7 +51,7 @@ cleanup_fn rm -f $files
 do_test ()
 {
     # We use jq to normalize the output and convert it to plain text.
-    nbdkit -U - data data="$1" size="$2" \
+    nbdkit -U - data "$1" size="$2" \
            --run 'qemu-img map -f raw --output=json $nbd' |
         jq -c '.[] | {start:.start, length:.length, data:.data, zero:.zero}' \
            > $out
