@@ -38,7 +38,7 @@ set -e
 set -x
 
 requires qemu-img --version
-requires nbdsh -c 'exit (not h.supports_uri ())'
+requires nbdsh -c 'exit(not h.supports_uri())'
 
 function do_test_info ()
 {
@@ -49,13 +49,23 @@ function do_test_info ()
 function do_test_read512 ()
 {
     nbdkit -U - --filter=offset --filter=truncate pattern size=1024 \
-           "$@" --run 'nbdsh -u "$uri" -c "h.pread (512, 0)"'
+           "$@" --run 'nbdsh -u "$uri" -c "
+try:
+  h.pread(512, 0)
+except nbd.Error:
+  exit(1)
+"'
 }
 
 function do_test_zero512 ()
 {
     nbdkit -U - --filter=offset --filter=truncate memory size=1024 \
-           "$@" --run 'nbdsh -u "$uri" -c "h.zero (512, 0)"'
+           "$@" --run 'nbdsh -u "$uri" -c "
+try:
+  h.zero(512, 0)
+except nbd.Error:
+  exit (1)
+"'
 }
 
 function expect_fail ()
