@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2018-2019 Red Hat Inc.
+# Copyright (C) 2018-2020 Red Hat Inc.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -93,7 +93,7 @@ esac
 EOF
 
 # Multi-line message, ESHUTDOWN
-do_test 'Cannot send after transport endpoint shutdown' <<'EOF'
+do_test "(Cannot|Can't) send after (transport endpoint|socket) shutdown" <<'EOF'
 case "$1" in
     get_size) echo 1M ;;
     pread) printf 'ESHUTDOWN\nline one\nline two\n' >&2; exit 1 ;;
