@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2017 Red Hat Inc.
+ * Copyright (C) 2017-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -58,6 +58,14 @@
 #include <sys/un.h>
 #include <sys/wait.h>
 
+#include "byte-swapping.h"
+#include "nbd-protocol.h"
+
+#ifndef SOCK_CLOEXEC
+/* For this file, we don't care if fds are marked cloexec; leaking is okay.  */
+#define SOCK_CLOEXEC 0
+#endif
+
 #define FIRST_SOCKET_ACTIVATION_FD 3
 
 #define NBDKIT_START_TIMEOUT 30 /* seconds */
@@ -79,32 +87,9 @@ static pid_t pid = 0;
 static void
 cleanup (void)
 {
-  int status;
-
-  if (pid > 0) {
+  if (pid > 0)
     kill (pid, SIGTERM);
 
-    /* Check the status of nbdkit is normal on exit. */
-    if (waitpid (pid, &status, 0) == -1) {
-      perror ("waitpid");
-      _exit (EXIT_FAILURE);
-    }
-    if (WIFEXITED (status) && WEXITSTATUS (status) != 0) {
-      _exit (WEXITSTATUS (status));
-    }
-    if (WIFSIGNALED (status)) {
-      /* Note that nbdkit is supposed to catch the signal we send and
-       * exit cleanly, so the following shouldn't happen.
-       */
-      fprintf (stderr, "nbdkit terminated by signal %d\n", WTERMSIG (status));
-      _exit (EXIT_FAILURE);
-    }
-    if (WIFSTOPPED (status)) {
-      fprintf (stderr, "nbdkit stopped by signal %d\n", WSTOPSIG (status));
-      _exit (EXIT_FAILURE);
-    }
-  }
-
   unlink (pidpath);
   unlink (sockpath);
   rmdir (tmpdir);
@@ -117,7 +102,7 @@ main (int argc, char *argv[])
   struct sockaddr_un addr;
   char pid_str[16];
   size_t i, len;
-  char magic[8];
+  uint64_t magic;
 
   if (mkdtemp (tmpdir) == NULL) {
     perror ("mkdtemp");
@@ -215,12 +200,7 @@ main (int argc, char *argv[])
    * not from the path), so we should be able to connect to the Unix
    * domain socket by its path and receive an NBD magic string.
    */
-#ifdef SOCK_CLOEXEC
   sock = socket (AF_UNIX, SOCK_STREAM|SOCK_CLOEXEC, 0);
-#else
-  sock = socket (AF_UNIX, SOCK_STREAM, 0);
-  fcntl (sock, F_SETFD, FD_CLOEXEC);
-#endif
   if (sock == -1) {
     perror ("socket");
     exit (EXIT_FAILURE);
@@ -232,12 +212,12 @@ main (int argc, char *argv[])
     exit (EXIT_FAILURE);
   }
 
-  if (read (sock, magic, 8) != 8) {
+  if (read (sock, &magic, sizeof magic) != sizeof magic) {
     perror ("read");
     exit (EXIT_FAILURE);
   }
 
-  if (memcmp (magic, "NBDMAGIC", 8) != 0) {
+  if (be64toh (magic) != NBD_MAGIC) {
     fprintf (stderr, "%s FAILED: did not read magic string from server\n",
              program_name);
     exit (EXIT_FAILURE);
