@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2017-2018 Red Hat Inc.
-# All rights reserved.
+# Copyright (C) 2017-2020 Red Hat Inc.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -35,11 +34,7 @@ source ./functions.sh
 set -x
 set -e
 
-# Check qemu-img exists.
-if ! qemu-img --version; then
-    echo "$0: missing qemu-img"
-    exit 77
-fi
+requires qemu-img --version
 
 files="test-zero.out"
 rm -f $files
